"{ Package: 'cvut:stx/goodies/libsvn' }"

"{ NameSpace: SVN }"

Task subclass:#FileoutLikeTask
	instanceVariableNames:'packageClassesChanged packageExtensionsChanged extensionMethods'
	classVariableNames:''
	poolDictionaries:''
	category:'SVN-Tasks'
!


!FileoutLikeTask methodsFor:'accessing'!

extensionMethods
    ^ extensionMethods
!

extensionMethods:aCollection"of Methods"

    extensionMethods := aCollection.

    "Modified: / 14-05-2009 / 13:38:15 / Jan Vrany <vranyj1@fel.cvut.cz>"
! !

!FileoutLikeTask methodsFor:'executing - private'!

doAddNewContainers
    | filesToAdd  containers |

    SVN::ActivityNotification notify: 'Adding new containers'.
    containers := self isSelectiveFileoutTask 
                ifTrue: [ #() ]
                ifFalse: [ self containersToFileOut ].
    filesToAdd := ((StatusCommand new)
                workingCopy: self workingCopy;
                paths: containers;
                execute) select: [:wcEntry | wcEntry status isUnversioned ]
                    thenCollect: [:wcEntry | wcEntry path ].
    filesToAdd isEmpty 
        ifFalse: 
            [ (AddCommand new)
                workingCopy: self workingCopy;
                paths: filesToAdd;
                execute.
            (PropsetCommand new)
                workingCopy: self workingCopy;
                name: 'svn:keywords';
                value: 'Id';
                paths: filesToAdd;
                execute.
            (PropsetCommand new)
                workingCopy: self workingCopy;
                name: 'svn:eol-style';
                value: 'LF';
                paths: filesToAdd;
                execute ]

    "Created: / 11-04-2008 / 10:58:53 / Jan Vrany <vranyj1@fel.cvut.cz>"
    "Modified: / 19-08-2009 / 14:00:26 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

doCompileVersionMethods
    | klasses versionMethodName |

    versionMethodName := SVNSourceCodeManager nameOfVersionMethodInClasses.

    SVN::ActivityNotification notify: 'Compiling #version methods...'.
    klasses := self classesToFileOut asArray.
    klasses withIndexDo: 
            [:cls :index | 
            | metaCls |

            metaCls := cls theMetaclass.
            (metaCls includesSelector: versionMethodName) 
                ifFalse: 
                    [ metaCls compile: (metaCls versionMethodTemplateForSourceCodeManager:SVNSourceCodeManager) classified: 'documentation'.
                    (metaCls compiledMethodAt: versionMethodName) setPackage: self package asSymbol ].
            SVN::ProgressNotification 
                notify: ('Compiling %1 (%2)' bindWith:versionMethodName with:cls nameWithoutPrefix)
                progress: (klasses size / 100) * index. ]

    "Created: / 28-05-2008 / 07:43:43 / Jan Vrany <vranyj1@fel.cvut.cz>"
    "Modified: / 16-08-2009 / 12:59:50 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

doFileOutAll

    self
        doInitStateVariables;
        doUpdateProjectDefinition;
        doUpdateBuildSupportFiles;
        doCompileVersionMethods;
        doCompileSvnRevisionNrMethod:false;
        doFileOutPackageClasses;
        doFileOutPackageExtensions;
        doAddNewContainers;
        doRemoveOldContainers.

    "Created: / 17-08-2009 / 18:28:18 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

doFileOutPackageClasses
    | klasses |

    ActivityNotification notify: 'Filing out package classes'.
    klasses := self classesToFileOut asArray.
    klasses withIndexDo: 
            [:cls :index | 
            | clsStream |

            SVN::ProgressNotification notify: 'Filing out ' , cls nameWithoutPrefix
                progress: (100 / klasses size) * index.
            (self shouldFileOutClass: cls) 
                ifTrue: 
                    [ 
                    [ clsStream := self workingCopy containersWriteStreamForClass: cls.
                    self workingCopy fileOutClass: cls on: clsStream ] ensure: [ clsStream ifNotNil: [ clsStream close ] ] ] ]

    "Created: / 11-04-2008 / 10:58:16 / Jan Vrany <vranyj1@fel.cvut.cz>"
    "Modified: / 19-08-2009 / 13:44:00 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

doFileOutPackageExtensions
    | extensionsStream |

    self extensionMethodsToFileOut isEmpty ifTrue: [ ^ self ].
    SVN::ActivityNotification notify: 'Filing out extension methods'.
    extensionsStream := self workingCopy containerWriteStreamForExtensions.
    [ self workingCopy fileOutExtensionsOn: extensionsStream ] 
        ensure: [ extensionsStream close ]

    "Modified: / 07-04-2008 / 08:37:25 / janfrog"
    "Created: / 11-04-2008 / 10:58:36 / Jan Vrany <vranyj1@fel.cvut.cz>"
    "Modified: / 14-05-2009 / 13:37:40 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

doInitStateVariables
    packageClassesChanged := self workingCopy packageClassesChanged.
    packageExtensionsChanged := self workingCopy packageExtensionsChanged.

    "Created: / 17-06-2009 / 10:17:04 / Jan Vrany <vranyj1@fel.cvut.cz>"
    "Modified: / 13-08-2009 / 10:18:37 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

doRemoveOldContainers
    | containers  containersToKeep  containersToDelete |

    self isSelectiveFileoutTask ifTrue: [ ^ self ].
    SVN::ActivityNotification notify: 'Removing old containers'.
    containers := self workingCopy containers.
    containersToKeep := self workingCopy containersToKeep.
    containersToDelete := containers \ containersToKeep.
    containersToDelete isEmpty 
        ifFalse: 
            [ (DeleteCommand new)
                workingCopy: self workingCopy;
                paths: containersToDelete;
                execute ].

    "Created: / 11-04-2008 / 11:00:27 / Jan Vrany <vranyj1@fel.cvut.cz>"
    "Modified: / 19-08-2009 / 15:03:51 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

doUpdateBuildSupportFiles

    |pkgDef|
    SVN::ActivityNotification notify:'Updating build files'.
    pkgDef := self workingCopy packageDefinition.
    (self buildSupportFilesFor: pkgDef) do:
        [:supportFileName| |supportFile stream|
        supportFile := workingCopy path / supportFileName.
        (packageClassesChanged or:[supportFile exists not]) ifTrue:
            [SVN::ActivityNotification notify:'Updating ', supportFileName.
            [stream :=  supportFile writeStream.
            stream nextPutAll:(pkgDef generateFile:supportFileName)]
                ensure:[ stream ifNotNil:[ stream close ]]]].

    "Created: / 20-06-2009 / 16:01:16 / Jan Vrany <vranyj1@fel.cvut.cz>"
    "Modified: / 21-06-2009 / 15:18:17 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

doUpdateProjectDefinition
    |prjClass|

    (packageClassesChanged | packageExtensionsChanged) ifFalse:[^self].

    SVN::ActivityNotification notify:'Updating project definition'.
    prjClass := Smalltalk 
                at:(ProjectDefinition initialClassNameForDefinitionOf:self package) 
                        asSymbol.
    prjClass ifNotNil:
        [Class packageQuerySignal 
            answer:prjClass package
            do:
                [prjClass theNonMetaclass 
                    forEachContentsMethodsCodeToCompileDo:
                        [:code :category | 
                        prjClass theMetaclass 
                            compile:code classified:category]
                            ignoreOldDefinition:true]]

    "Created: / 11-04-2008 / 11:01:06 / Jan Vrany <vranyj1@fel.cvut.cz>"
    "Modified: / 13-08-2009 / 09:13:14 / Jan Vrany <vranyj1@fel.cvut.cz>"
! !

!FileoutLikeTask methodsFor:'private'!

classesToFileOut
    ^ self workingCopy packageClassesFiltered: 
            [:class | 
            class isLoaded and: [ classes isNil or: [ classes includes: class ] ] ]

    "Created: / 23-03-2009 / 12:08:24 / Jan Vrany <vranyj1@fel.cvut.cz>"
    "Modified: / 11-06-2009 / 18:17:38 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

containersToFileOut
    | containers |

    containers := self classesToFileOut 
                collect: [:class | self repository containerNameForClass: class ].
    self extensionMethodsToFileOut isEmpty 
        ifFalse: [ containers add: self repository containerNameForExtensions ].
    ^ containers

    "Created: / 14-05-2009 / 11:35:05 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

extensionMethodsToFileOut
    ^ self workingCopy packageExtensionsFiltered: 
            [:mth | 
            extensionMethods isNil or: [ extensionMethods includes: mth ] ]

    "Created: / 14-05-2009 / 11:32:01 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

shouldFileOutClass: class

    "Do not fileout autoloaded classes, 
    they are untouched"
    class isLoaded ifFalse:[^false].

    class theNonMetaclass 
        methodsDo:[:mth|(self shouldFileOutMethod: mth) ifTrue:[^true]].
    class theMetaclass 
        methodsDo:[:mth|(self shouldFileOutMethod: mth) ifTrue:[^true]].
    ^false

    "
        CommitTask basicNew 
            package: #'stx:goodies/libsvn';
            shouldFileOutClass: CommitTask

        CommitTask basicNew 
            package: #'stx:libbasic';
            shouldFileOutClass: Object      
    "

    "Created: / 24-06-2009 / 19:04:48 / Jan Vrany <vranyj1@fel.cvut.cz>"
    "Modified: / 19-08-2009 / 13:54:39 / Jan Vrany <vranyj1@fel.cvut.cz>"
!

shouldFileOutMethod: mth

    ^mth package = package and:
        [mth getSourcePosition isNil]

    "
        CommitTask basicNew 
            package: #'stx:goodies/libsvn';
            shouldFileOutMethod: (CommitTask >> #shouldFileOutMethod:)

        CommitTask basicNew 
            package: #'stx:libbasic';
            shouldFileOutMethod: (Object >> #yourself)
    "

    "Created: / 24-06-2009 / 19:07:27 / Jan Vrany <vranyj1@fel.cvut.cz>"
! !

!FileoutLikeTask methodsFor:'queries'!

isSelectiveFileoutTask
    ^ classes notNil or: [ extensionMethods notNil ]

    "Created: / 14-05-2009 / 13:24:52 / Jan Vrany <vranyj1@fel.cvut.cz>"
! !

!FileoutLikeTask class methodsFor:'documentation'!

version
    ^ '$Header$'
!

version_CVS
    ^ '$Header$'
!

version_SVN
    ^'Id: SVN__FileoutLikeTask.st 110 2009-08-19 13:21:10Z vranyj1 '
! !
